==> katana_routing-0.8.4%2B4.tar.gz: lib/src/extensions.dart <==
part of katana_routing;

extension NavigatorStateExtensions on NavigatorState {
  /// Pops the page to the page with the specified path.
  ///
  /// Enter the pathname of the page in [name].
  Future<void> popUntilNamed(String name) async {
    popUntil((route) {
      return route.settings.name == name;
    });
  }

  /// Reset all history and go to a specific page.
  ///
  /// After all pages are [pop()],
  /// the page specified by [newRouteName] is [pushNamed()].
  ///
  /// In [arguments], put [RouteQuery] and so on.
  ///
  /// Ideal for moving after logging out.
  Future<T?> resetAndPushNamed<T extends Object?>(
    String newRouteName, {
    Object? arguments,
  }) {
    return pushNamedAndRemoveUntil(
      newRouteName,
      (route) => false,
      arguments: arguments,
    );
  }
}

extension BuildContextExtensions on BuildContext {
  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuild() {
    UIScope.of(this).rebuild();
  }

  /// Rebuild all active pages.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildAll() {
    UIScope.of(this).rebuildAll();
  }

  /// Rebuild only the pages that are `true` in [test].
  ///
  /// Normally, you should check if you want to rebuild based on the page name.
  /// ```
  /// context.rebuildWhere((page) {
  ///   return page.routeSettings?.name == "/";
  /// });
  /// ```
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuildWhere(bool Function(PageState page) test) {
    UIScope.of(this).rebuildWhere(test);
  }

  /// Open a new URL or screen.
  ///
  /// If [path] starts with `http`,
  /// it opens a browser, otherwise it opens a screen path.
  ///
  /// It is possible to configure the settings for
  /// opening the screen by specifying [query].
  Future<T?> open<T extends Object?>(String path, [RouteQuery? query]) async {
    if (path.startsWith("http")) {
      await openURL(path);
      return null;
    } else {
      return await navigator.pushNamed(path, arguments: query);
    }
  }

  /// Outputs the theme related to the context.
  ThemeData get theme => Theme.of(this);

  /// Outputs the widget theme related to the context.
  WidgetTheme get widgetTheme => WidgetTheme.of(this);

  /// Get the Navigator related to context.
  NavigatorState get navigator => Navigator.of(this);

  /// Get the Root navigator related to context.
  NavigatorState get rootNavigator => Navigator.of(this, rootNavigator: true);

  /// Get the Flavor.
  String get flavor => FlavorScope.of(this).flavor;

  /// Get the Design type.
  DesignType get designType => DesignTypeScope.of(this).designType;

  /// If you want to use the style for web when you are on the web, use `true`.
  bool get webStyle => DesignTypeScope.of(this).webStyle;

  /// Get the route settins.
  RouteSettings get route =>
      ModalRoute.of(this)?.settings ?? const RouteSettings();

  /// Get the data passed to the page.
  ///
  /// Basically it returns the [RouteQuery] data passed from the previous page.
  ///
  /// If the [RouteQuery] data is not found,
  /// the map stored in [PageHookWidget] is retrieved.
  DynamicMap get arg =>
      ModalRoute.of(this)?.settings.arguments as DynamicMap? ??
      UIScope.of(this)._map;

  /// Gets the value from the [key] of the map stored in [arg].
  ///
  /// If [key] does not exist in the map, [orElse] is returned.
  T get<T>(String key, T orElse) {
    return arg.get(key, orElse);
  }

  /// Get the list corresponding to [key] in the map.
  ///
  /// If [key] is not found, the list of [orElse] is returned.
  List<T> getAsList<T>(String key, [List<T>? orElse]) {
    if (!arg.containsKey(key)) {
      return orElse ?? [];
    }
    return (arg[key] as List?)?.cast<T>() ?? orElse ?? [];
  }

  /// Get the map corresponding to [key] in the map.
  ///
  /// If [key] is not found, the map of [orElse] is returned.
  Map<String, T> getAsMap<T>(String key, [Map<String, T>? orElse]) {
    if (!arg.containsKey(key)) {
      return orElse ?? {};
    }
    return (arg[key] as Map?)?.cast<String, T>() ?? orElse ?? {};
  }

  /// Get the set corresponding to [key] in the map.
  ///
  /// If [key] is not found, the set of [orElse] is returned.
  Set<T> getAsSets<T>(String key, [Set<T>? orElse]) {
    if (!arg.containsKey(key)) {
      return orElse ?? {};
    }
    return (arg[key] as Set?)?.cast<T>() ?? orElse ?? {};
  }

  /// Get the set corresponding to [key] in the DateTime.
  ///
  /// If [key] is not found, the set of [orElse] is returned.
  DateTime getAsDateTime(String key, [DateTime? orElse]) {
    if (!arg.containsKey(key)) {
      return orElse ?? DateTime.now();
    }
    final millisecondsSinceEpoch = arg[key] as int?;
    if (millisecondsSinceEpoch == null) {
      return orElse ?? DateTime.now();
    }
    return DateTime.fromMillisecondsSinceEpoch(millisecondsSinceEpoch);
  }

  /// Get [value] to [key] of the map stored in [arg].
  ///
  /// The data will be unique within the page and can continue to be used even if the widget is rebuilt.
  dynamic operator [](Object? key) => arg[key];

  /// Save [value] to [key] of the map stored in [arg].
  ///
  /// The data will be unique within the page and can continue to be used even if the widget is rebuilt.
  void operator []=(String key, dynamic value) {
    arg[key] = value;
  }

  /// Get the media qury related to context
  MediaQueryData get mediaQuery => MediaQuery.of(this);

  /// Releases focus such as text field.
  void unfocus() {
    FocusManager.instance.primaryFocus?.unfocus();
  }
}

extension RoutingDynamicMapExtensions on DynamicMap {
  /// Convert the theme color from [DynamicMap].
  ThemeColor? toThemeColor() {
    return ThemeColor._fromMap(this);
  }
}

extension ThemeExtension on ThemeData {
  /// Text color.
  Color get textColor {
    return colorScheme.onBackground;
  }

  /// Text color on primary.
  Color get textColorOnPrimary {
    return colorScheme.onPrimary;
  }

  /// Text color on secondary.
  Color get textColorOnSecondary {
    return colorScheme.onSecondary;
  }

  /// Text color on surface.
  Color get textColorOnSurface {
    return colorScheme.onSurface;
  }

  /// Text color on error.
  Color get textColorOnError {
    return colorScheme.onError;
  }
}

==> katana_routing-0.9.0%2B4.tar.gz: lib/src/extensions.dart <==
part of katana_routing;

extension NavigatorStateExtensions on NavigatorState {
  /// Pops the page to the page with the specified path.
  ///
  /// Enter the pathname of the page in [name].
  Future<void> popUntilNamed(String name) async {
    popUntil((route) {
      return route.settings.name == name;
    });
  }

  /// Reset all history and go to a specific page.
  ///
  /// After all pages are [pop()],
  /// the page specified by [newRouteName] is [pushNamed()].
  ///
  /// In [arguments], put [RouteQuery] and so on.
  ///
  /// Ideal for moving after logging out.
  Future<T?> resetAndPushNamed<T extends Object?>(
    String newRouteName, {
    Object? arguments,
  }) {
    return pushNamedAndRemoveUntil(
      newRouteName,
      (route) => false,
      arguments: arguments,
    );
  }
}

extension BuildContedxtExtensions on BuildContext {
  DynamicMap get _arg {
    final pageScoped = _PageScoped.of(this);
    return pageScoped._modalRoute?.settings.arguments as DynamicMap? ??
        pageScoped._map;
  }

  /// Outputs the theme related to the context.
  ThemeData get theme => Theme.of(this);

  /// Outputs the widget theme related to the context.
  WidgetTheme get widgetTheme => WidgetTheme.of(this);

  /// Get the Navigator related to context.
  NavigatorState get navigator => Navigator.of(this);

  /// Get the Root navigator related to context.
  NavigatorState get rootNavigator => Navigator.of(this, rootNavigator: true);

  /// Get the Flavor.
  String get flavor => FlavorScope.of(this).flavor;

  /// Get the Design type.
  DesignType get designType => DesignTypeScope.of(this).designType;

  /// If you want to use the style for web when you are on the web, use `true`.
  bool get webStyle => DesignTypeScope.of(this).webStyle;

  /// Get the route settins.
  RouteSettings get route =>
      ModalRoute.of(this)?.settings ?? const RouteSettings();

  /// Get the media qury related to context
  MediaQueryData get mediaQuery => MediaQuery.of(this);

  /// Releases focus such as text field.
  void unfocus() {
    FocusManager.instance.primaryFocus?.unfocus();
  }

  /// True if the specified [key] is contained in scoped.
  bool containsKey(Object? key) {
    return _arg.containsKey(key);
  }

  /// True if the specified [value] is contained in scoped.
  bool containsValue(Object? value) {
    return _arg.containsValue(value);
  }

  /// Gets the value from the [key] of the map stored in scoped.
  ///
  /// If [key] does not exist in the map, [orElse] is returned.
  T get<T>(String key, T orElse) {
    return _arg.get(key, orElse);
  }

  /// Get the list corresponding to [key] in the map.
  ///
  /// If [key] is not found, the list of [orElse] is returned.
  List<T> getAsList<T>(String key, [List<T>? orElse]) {
    if (!_arg.containsKey(key)) {
      return orElse ?? [];
    }
    return (_arg[key] as List?)?.cast<T>() ?? orElse ?? [];
  }

  /// Get the map corresponding to [key] in the map.
  ///
  /// If [key] is not found, the map of [orElse] is returned.
  Map<String, T> getAsMap<T>(String key, [Map<String, T>? orElse]) {
    if (!_arg.containsKey(key)) {
      return orElse ?? {};
    }
    return (_arg[key] as Map?)?.cast<String, T>() ?? orElse ?? {};
  }

  /// Get the set corresponding to [key] in the map.
  ///
  /// If [key] is not found, the set of [orElse] is returned.
  Set<T> getAsSets<T>(String key, [Set<T>? orElse]) {
    if (!_arg.containsKey(key)) {
      return orElse ?? {};
    }
    return (_arg[key] as Set?)?.cast<T>() ?? orElse ?? {};
  }

  /// Get the set corresponding to [key] in the DateTime.
  ///
  /// If [key] is not found, the set of [orElse] is returned.
  DateTime getAsDateTime(String key, [DateTime? orElse]) {
    if (!_arg.containsKey(key)) {
      return orElse ?? DateTime.now();
    }
    final millisecondsSinceEpoch = _arg[key] as int?;
    if (millisecondsSinceEpoch == null) {
      return orElse ?? DateTime.now();
    }
    return DateTime.fromMillisecondsSinceEpoch(millisecondsSinceEpoch);
  }

  /// Get [value] to [key] of the map stored in [arg].
  ///
  /// The data will be unique within the page and can continue to be used even if the widget is rebuilt.
  dynamic operator [](Object? key) {
    return _arg[key];
  }

  /// Save [value] to [key] of the map stored in [arg].
  ///
  /// The data will be unique within the page and can continue to be used even if the widget is rebuilt.
  void operator []=(String key, dynamic value) {
    _arg[key] = value;
  }
}

extension WidgetRefExtensions on WidgetRef {
  DynamicMap get _arg {
    final scoped = Scoped.of(this as BuildContext);
    return scoped._map;
  }

  /// Explicitly rebuild the widget.
  ///
  /// Basically, do not use it.
  ///
  /// Use this only if you want to force a rebuild.
  void rebuild() {
    Scoped.of(this as BuildContext).rebuild();
  }

  /// Open a new URL or screen.
  ///
  /// If [path] starts with `http`,
  /// it opens a browser, otherwise it opens a screen path.
  ///
  /// It is possible to configure the settings for
  /// opening the screen by specifying [query].
  Future<T?> open<T extends Object?>(String path, [RouteQuery? query]) async {
    if (path.startsWith("http")) {
      await openURL(path);
      return null;
    } else {
      return await (this as BuildContext)
          .navigator
          .pushNamed(path, arguments: query);
    }
  }

  /// A builder for creating and saving scoped value.
  ///
  /// If [key] is specified, the data will be saved with that key.
  ///
  /// Since it is not usually used directly from [get], [prefix] is given to prevent duplication of values.
  ///
  /// It creates a scoped value from [builder].
  ///
  /// If [test] is `true` or the scoped value for the Key is empty, update the value.
  @protected
  TResult valueBuilder<TResult, TScopedValue extends ScopedValue<TResult>>({
    required String key,
    required TScopedValue Function() builder,
    String prefix = r"_$",
  }) {
    key = "$prefix$key";
    final found = _arg.get<ScopedValueState<TResult, TScopedValue>?>(key, null);
    final value = builder.call();
    if (found != null) {
      final oldValue = found.value;
      found._ref = this;
      found._value = value;
      found.didUpdateValue(oldValue);
      return found.build();
    } else {
      final state = value.createState();
      state._ref = this;
      state._value = value;
      state.initValue();
      _arg[key] = state;
      return state.build();
    }
  }
}

extension ThemeExtension on ThemeData {
  /// Text color.
  Color get textColor {
    return colorScheme.onBackground;
  }

  /// Text color on primary.
  Color get textColorOnPrimary {
    return colorScheme.onPrimary;
  }

  /// Text color on secondary.
  Color get textColorOnSecondary {
    return colorScheme.onSecondary;
  }

  /// Text color on surface.
  Color get textColorOnSurface {
    return colorScheme.onSurface;
  }

  /// Text color on error.
  Color get textColorOnError {
    return colorScheme.onError;
  }

  /// Secondary color.
  Color get secondaryColor {
    return colorScheme.secondary;
  }

  /// Surface color.
  Color get surfaceColor {
    return colorScheme.surface;
  }
}
